<template>
  <div class="add-quotation">
    <el-dialog
      :title="editCommonConfig.title + '销售子单'"
      :visible.sync="editConfig.visible"
      :width="dialogWidth"
      :before-close="handleClose"
      append-to-body
      custom-class="iframe-dialog"
    >
      <el-form
        v-if="refreshFlag"
        ref="form"
        :model="editConfig.infomation"
        :rules="rules"
        label-position="right"
        label-width="100px"
        size="mini"
      >
        <!-- 信息 -->
        <div class="basic-info">
          <!-- 基本信息 -->
          <div class="basic-info-title">基本信息</div>
          <div class="basic-info-view">
            <el-row>
              <el-col :span="12">
                <el-form-item label="单据编号" prop="number">
                  <WordInput
                    v-if="codenumer && (explain != '' || isIdDisabled) && method == 0"
                    :codenumer="codenumer"
                    :sum="sum"
                    :disabled="editConfig.title !== '新建' || isIdDisabled"
                    :inputValue="inputValue"
                    @codeList="codeList"
                  />
                  <span v-else-if="method == 0" style="color: #f56c6c"
                    >请优先配置编码规范 <el-button type="text" @click="numberClick"> 配置规范 </el-button></span
                  >
                  <span v-else>{{ editConfig.title === "新建" ? "自动生成" : editConfig.infomation.number }}</span>
                </el-form-item>
              </el-col>
              <el-col :span="12">
                <el-form-item label="客户名称" prop="client_name">
                  <div class="custom-name">
                    <el-autocomplete
                      v-model="editConfig.infomation.client_name"
                      :fetch-suggestions="
                        (queryString, callback) => {
                          querySearchAsync(queryString, callback, 'client')
                        }
                      "
                      value-key="name"
                      @select="handleSelectClient('client', $event)"
                    ></el-autocomplete>
                    <div class="common-select-btn" @click="selClientClick('client')">
                      <i class="el-icon-circle-plus-outline" title="选择"></i>
                    </div>
                    <div
                      v-if="editConfig.infomation.client_name && editConfig.infomation.client_name.length > 0"
                      class="common-select-btn"
                      @click="clearupClient('client')"
                    >
                      <i class="el-icon-remove-outline" title="清除"></i>
                    </div>
                  </div>
                </el-form-item>
              </el-col>

              <el-col :span="12">
                <el-form-item label="销售总单" prop="master_order_number">
                  <div class="custom-name">
                    <el-autocomplete
                      v-model="editConfig.infomation.master_order_number"
                      :fetch-suggestions="
                        (queryString, callback) => {
                          querySearchAsync(queryString, callback, 'master')
                        }
                      "
                      value-key="number"
                      @select="handleSelectClient('master', $event)"
                      placeholder="请先选择客户名称"
                      :disabled="isAddMaster"
                    ></el-autocomplete>
                    <div v-if="!isAddMaster" class="common-select-btn" @click="selClientClick('master')">
                      <i class="el-icon-circle-plus-outline" title="选择"></i>
                    </div>
                    <div
                      v-if="
                        editConfig.infomation.master_order_number &&
                        editConfig.infomation.master_order_number.length > 0 &&
                        !isAddMaster
                      "
                      class="common-select-btn"
                      @click="clearupClient('master')"
                    >
                      <i class="el-icon-remove-outline" title="清除"></i>
                    </div>
                  </div>
                </el-form-item>
              </el-col>
              <el-col :span="12">
                <el-form-item label="负责人" prop="memberId">
                  <el-select
                    v-model="editConfig.infomation.memberId"
                    placeholder="请选择"
                    size="mini"
                    style="width: 100%"
                  >
                    <el-option v-for="item in memberOptions" :key="item.id" :label="item.username" :value="item.id">
                    </el-option>
                  </el-select>
                </el-form-item>
              </el-col>
            </el-row>
          </div>
          <!-- 附件信息 -->
          <!-- <div class="basic-info-title">附件信息</div>
          <div class="basic-info-view">
            <el-row>
              <el-col :span="12">
                <el-form-item label="附件" prop="">
                  <template slot="label">
                    <div style="display: flex; float: right">
                      <div style="font-size: 16px">
                        <i class="el-icon-warning-outline" title="最多上传20个附件，最大限制5MB"></i>
                      </div>
                      <span style="margin-left: 5px">附件</span>
                    </div>
                  </template>
                  <div class="annex-view">
                    <div @click="addAnnexClick">
                      <div style="display: flex; float: right">
                        <div style="font-size: 16px"><i class="el-icon-paperclip"></i></div>
                        <span>添加</span>
                      </div>
                    </div>
                    <div class="setFormat" @click="setFormatClick">设置允许上传的文件格式</div>
                  </div>
                </el-form-item>
              </el-col>
            </el-row>
          </div> -->
          <!-- 产品管理 -->
          <div class="basic-info-title" style="display: flex">
            产品管理
            <div style="margin-left: 400px">
              币种
              <el-select v-model="editConfig.infomation.currency" placeholder="请选择" size="mini" style="width: 63%">
                <el-option v-for="item in currencyOptions" :key="item.id" :label="item.name" :value="item.id">
                </el-option>
              </el-select>
            </div>
          </div>
          <div class="product-view">
            <CommonFormTableView
              :show-summary="showSummary"
              :product-table-list="productTableList"
              @inputContent="inputContent"
              @addProductClick="addProductClick"
              @emptyProductClick="emptyProductClick"
              @recalculateProductClick="recalculateProductClick"
              @clearupProduct="clearupProduct"
            />
          </div>
        </div>
      </el-form>
      <div slot="footer" class="dialog-footer">
        <el-button type="primary" size="small" @click="saveClick('form')">保存</el-button>
        <el-button size="small" @click="editConfig.visible = false">取消</el-button>
      </div>
      <!-- 选择客户 -->
      <SelectClientDialog
        v-if="editSelectClientConfig.editVisible"
        :edit-common-config="editSelectClientConfig"
        @selClient="selClient"
      />
      <!-- 选择销售总单 -->
      <SelectCommonDialog
        v-if="editSelectMasterConfig.editVisible"
        :edit-common-config="editSelectMasterConfig"
        @selClient="selClient"
      />
    </el-dialog>
  </div>
</template>

<script>
import CommonFormTableView from "@/components/makepager/CommonFormTableView"
import { getAllData } from "@/api/client/client"
import { getAddSubOrder, getUpdateSubOrder } from "@/api/sales/subOrder"
import SelectClientDialog from "@/views/other/commonDialog/SelectClientDialog"
import SelectCommonDialog from "@/views/other/commonDialog/SelectCommonDialog"
import WordInput from "@/components/wordInput.vue"
import codeMixin from "@/views/client/followupRecords/mixin/codeMixin"
import { getMasterOrderList } from "@/api/sales/masterOrder"

export default {
  name: "SubOrderDialog",
  mixins: [codeMixin],
  components: { WordInput, CommonFormTableView, SelectClientDialog, SelectCommonDialog },
  props: {
    editCommonConfig: {
      type: Object,
      default: () => {
        return {
          visible: false,
          title: "新建",
          infomation: {}
        }
      }
    }
  },
  computed: {
    clientList() {
      return this.$store.state.getClientName.clientList
    }
    // masterOrderList() {
    //   return this.$store.state.getClientName.masterOrderList
    // }
  },
  data() {
    return {
      refreshFlag: true,
      dialogWidth: "50%",
      editConfig: this.editCommonConfig,
      rules: {
        number: [{ required: true, message: "请输入", trigger: "blur" }],
        master_order_number: [{ required: true, message: "请选择或创建销售总单", trigger: "blur" }],
        client_name: [{ required: true, message: "请选择", trigger: "change" }],
        memberId: [{ required: true, message: "请选择", trigger: "change" }]
      },
      memberOptions: [],
      discount: "2",
      currencyOptions: [{ id: 1, name: "人民币" }], // 币种
      discountOptions: [
        { value: "1", label: "百分比降价" },
        { value: "2", label: "直接降价" }
      ], // 整单折扣
      adjust: "1",
      adjustOptions: [
        { value: "1", label: "增加" },
        { value: "2", label: "减少" }
      ], // 调整
      productTableList: {},
      showSummary: {
        show: true,
        total: true,
        sumProp: ["Amount", "total"],
        mergeNumber: 2
      },
      editSelectClientConfig: {
        editVisible: false,
        title: "",
        infomation: {}
      },
      editSelectMasterConfig: {
        editVisible: false,
        title: "",
        infomation: {}
      },
      clientId: this.editCommonConfig.infomation.client_id,
      masterId: this.editCommonConfig.infomation.masterOrderId,
      tableData: [],
      productId: 1,
      addProductArr: [],
      objCode: { name: "", page: 0, pageSize: 0, type: "销售子单编码", codeStandID: "" },
      isAddMaster: true,
      masterOrderList: []
    }
  },
  created() {
    this.$store.dispatch("geClient")
    // this.$store.dispatch("geMaster")
    this.setTableForm()
    this.getCommonData()
    this.formInfo()
  },
  watch: {
    "editConfig.visible"(val) {
      if (val) {
        this.formInfo()
      }
    },
    "editConfig.infomation.codeStandID"() {
      this.formInfo()
    }
  },
  methods: {
    formInfo() {
      this.objCode.codeStandID = ""
      if (this.editConfig.infomation.codeStandID) {
        this.objCode.codeStandID = this.editConfig.infomation.codeStandID
      }
      this.getRCodeStandardList()
    },
    getCommonData() {
      getAllData()
        .then((res) => {
          console.log(res)
          if (res.code === 200) {
            this.memberOptions = res.data.member
            this.dataProcess()
          }
        })
        .catch((err) => {
          console.log(err)
        })
    },
    dataProcess() {
      this.masterOrderList.map((item) => {
        if (this.masterId === item.id) {
          this.editConfig.infomation.master_order_number = item.number
        }
      })
    },
    /**
     * 强制刷新form组件
     * 外部更改表单值不会触发表单校验,导致即使有值夜被表单校验阻塞住
     */
    refresh() {
      this.refreshFlag = false
      requestAnimationFrame(() => {
        this.refreshFlag = true
      })
    },
    // 保存
    saveClick(formName) {
      this.$refs[formName].validate((valid) => {
        if (valid) {
          console.log(this.editConfig.infomation)
          for (let i = 0; i < this.tableData.length; i++) {
            if (this.tableData[i].name.length === 0) {
              this.isNoProduct = true
              break
            } else {
              this.isNoProduct = false
            }
          }
          if (this.isNoProduct) {
            this.$message.error("产品名称不能为空")
          } else {
            const params = this.saveParams()
            console.log(params)
            if (this.editConfig.title === "新建") {
              getAddSubOrder(params).then((res) => {
                console.log(res)
                this.editConfig.visible = false
                if (res.code === 200) {
                  this.$message.success("添加成功")
                  this.$parent.getData()
                }
              })
            } else {
              getUpdateSubOrder(params).then((res) => {
                console.log(res)
                this.editConfig.visible = false
                if (res.code === 200) {
                  this.$message.success("编辑成功")
                  this.$parent.getData()
                }
              })
            }
          }
        } else {
          console.log("error submit")
          return false
        }
      })
    },
    saveParams() {
      let data = this.editConfig.infomation
      let params = {
        id: this.editConfig.title === "新建" ? 0 : data.id,
        clientId: this.clientId || 0,
        masterOrderId: this.masterId || 0,
        memberId: data.memberId || 0,
        number: data.number || "",
        product: this.tableData,
        codeStandID: data.ID,
        codeRule: this.codeRule
      }
      return params
    },
    handleClose() {
      this.editConfig.visible = false
    },
    // 获取销售总单
    getMasterList(id) {
      getMasterOrderList({
        page: 0,
        pageSize: 0,
        search_map: {
          client_id: id
        }
      }).then((res) => {
        if (res.code == 200) {
          this.masterOrderList = res.data.list
        }
      })
    },
    // 选择客户相关方法
    querySearchAsync(queryString, cb, value) {
      var restaurants = []
      if (value === "client") {
        restaurants = this.clientList
      } else if (value === "master") {
        restaurants = this.masterOrderList
      }
      var results = queryString ? restaurants.filter(this.createStateFilter(queryString, value)) : restaurants
      cb(results)
    },
    createStateFilter(queryString, value) {
      return (state) => {
        console.log(state)
        if (value === "client") {
          return state.name.toLowerCase().indexOf(queryString.toLowerCase()) === 0
        } else if (value === "master") {
          return state.number.toLowerCase().indexOf(queryString.toLowerCase()) === 0
        }
      }
    },
    handleSelectClient(value, item) {
      if (value === "client") {
        this.clientId = item.id
        this.isAddMaster = false
        this.getMasterList(item.id)
        this.editConfig.infomation.master_order_number = ""
        this.masterId = 0
      } else if (value === "master") {
        this.editConfig.infomation.master_order_number = item.number
        this.masterId = item.id
      }
      this.refresh()
    },
    selClientClick(value) {
      if (value == "client") {
        this.editSelectClientConfig.editVisible = true
      } else if (value == "master") {
        this.editSelectMasterConfig.title = "销售总单"
        this.editSelectMasterConfig.editVisible = true
        this.editSelectMasterConfig.clientId = this.clientId
        this.editSelectMasterConfig.tableInfomation = [...this.masterOrderList]
      }
    },
    selClient(row, value) {
      if (value == "client") {
        this.editConfig.infomation.client_name = row.name
        this.editConfig.infomation.client_id = row.id
        this.clientId = row.id
        this.isAddMaster = false
        this.getMasterList(row.id)
        this.editConfig.infomation.master_order_number = ""
        this.masterId = 0
      } else if (value == "master") {
        this.editConfig.infomation.master_order_number = row.number
        this.masterId = row.id
      }
      this.refresh()
    },
    // 清除已选择用户
    clearupClient(value) {
      if (value == "client") {
        this.editConfig.infomation.client_name = ""
        this.editConfig.infomation.client_id = 0
        this.clientId = 0
        this.isAddMaster = true
        this.editConfig.infomation.master_order_number = ""
        this.masterId = 0
      } else if (value == "master") {
        this.editConfig.infomation.master_order_number = ""
        this.masterId = 0
      }
      this.refresh()
    },
    // 添加附件
    addAnnexClick() {},
    // 设置允许上传文件格式
    setFormatClick() {},
    setTableForm() {
      if (this.editConfig.title === "新建" || this.editConfig.infomation.products.length === 0) {
        this.tableData = [
          {
            productId: this.productId,
            id: 0,
            amount: 0,
            desc: "",
            name: "",
            number: "",
            price: 0,
            total: 0
          }
        ]
      } else {
        this.tableData = this.editConfig.infomation.products
        this.tableData.map((item, index) => {
          item.productId = index + 1
        })
      }
      this.productTableList = {
        tableData: this.tableData,
        tableColumn: [
          { label: "#", prop: "productId", width: 40 },
          { label: "产品名称", prop: "name", productName: true, isRequird: true },
          { label: "产品编号", prop: "number" },
          { label: "数量", prop: "amount", inputNumber: true, isRequird: true },
          { label: "销售单价", prop: "price", inputFloat: true },
          { label: "价税合计", prop: "total", inputFloat: true }
        ]
      }
    },
    // 产品列表输入
    inputContent(val, prop, row) {
      this.productId = row.productId
      this.tableData.map((item) => {
        if (item.productId === row.productId) {
          item[prop] = val
        }
      })
    },
    // 产品新增
    addProductClick() {
      this.productId++
      this.tableData.push({
        productId: this.productId,
        id: 0,
        amount: 0,
        desc: "",
        name: "",
        number: "",
        price: 0,
        total: 0
      })
      this.showSummary.show = true
    },
    //  产品清空
    emptyProductClick() {
      this.productId = 1
      this.tableData = [
        {
          productId: this.productId,
          id: 0,
          amount: 0,
          desc: "",
          name: "",
          number: "",
          price: 0,
          total: 0
        }
      ]
      this.productTableList.tableData = this.tableData
    },
    // 产品重算
    recalculateProductClick() {},
    clearupProduct(data) {
      this.tableData = data
      this.productTableList.tableData = this.tableData
    }
  }
}
</script>

<!-- Add "scoped" attribute to limit CSS to this component only -->
<style lang="scss" scoped>
::v-deep {
  .iframe-dialog .el-dialog__body {
    .basic-info {
      .basic-info-title {
        background-color: #f4f8fe;
        padding-left: 10px;
        font-size: 15px;
        font-weight: bold;
        color: #666;
        height: 42px;
        line-height: 42px;
      }
      .basic-info-view {
        margin-top: 10px;
        padding-right: 40px;
        .custom-name {
          display: flex;
          .common-select-btn {
            margin-left: 5px;
            font-size: 18px;
            cursor: pointer;
          }
          .el-autocomplete {
            flex: 1;
          }
        }
      }
      .address-view {
        margin-top: 10px;
        padding-right: 40px;
      }
      .annex-view {
        display: flex;
        color: #6166d3;
        .setFormat {
          margin-left: 10px;
        }
      }
    }
    .dialog-footer {
      background-color: #f5f5f5;
      height: 55px;
      line-height: 55px;
    }
  }
}
</style>
